/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLElement.h"
#include "HTMLSelectElement.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomHTMLElement.h"
#include "gdom/GdomHTMLSelectElement.h"
#include "gdom/GdomNode.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomHTMLElement.h"
#include "gdom/PrivateGdomHTMLSelectElement.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateIntGdomHTMLElement.h"
#include "gdom/PrivateIntGdomHTMLSelectElement.h"
#include "gdom/PrivateIntGdomNode.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLSelectElement::GDOMHTMLSelectElement(PassRefPtr<WebCore::HTMLSelectElement> impl)
    : GDOMHTMLElement(impl)
{
}

void
gdom_html_select_element_add (GdomHTMLSelectElement *thiz, GdomHTMLElement * element, GdomHTMLElement * before)
{
    g_return_if_fail (thiz);
    WebCore::HTMLSelectElement * item = (gdom_html_select_element_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (element);
    g_return_if_fail (before);
    WebCore::HTMLElement * _g_element = (gdom_html_element_get_private(element));
    g_return_if_fail (_g_element);
    WebCore::HTMLElement * _g_before = (gdom_html_element_get_private(before));
    g_return_if_fail (_g_before);
    WebCore::ExceptionCode ec = 0;
    item->add(_g_element, _g_before, ec);

}

void
gdom_html_select_element_remove (GdomHTMLSelectElement *thiz, glong index)
{
    g_return_if_fail (thiz);
    WebCore::HTMLSelectElement * item = (gdom_html_select_element_get_private(thiz));
    g_return_if_fail (item);
    item->remove(index);

}

GdomNode *
gdom_html_select_element_item (GdomHTMLSelectElement *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLSelectElement * item = (gdom_html_select_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->item(index));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_html_select_element_named_item (GdomHTMLSelectElement *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLSelectElement * item = (gdom_html_select_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->namedItem(_g_name));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLSelectElement.h"
#include <gdom/GdomHTMLSelectElement.h>
#include <gdom/PrivateGdomHTMLSelectElement.h>
#include <gdom/PrivateIntGdomHTMLSelectElement.h>
#include <gdom/GdomHTMLSelectElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLSelectElement, gdom_html_select_element, GDOM_TYPE_HTML_ELEMENT);

/*
struct _GdomHTMLSelectElementPrivate {
    class WebCore::HTMLSelectElement *item;
};

#define GDOM_HTML_SELECT_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_SELECT_ELEMENT, GdomHTMLSelectElementPrivate))
*/
G_END_DECLS
WebCore::HTMLSelectElement *
gdom_html_select_element_get_private(GdomHTMLSelectElement *request)
{
    GDOMHTMLSelectElement *res;
    res = static_cast<GDOMHTMLSelectElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLSelectElement*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_TYPE,
    PROP_SELECTED_INDEX,
    PROP_VALUE,
    PROP_LENGTH,
    PROP_FORM,
    PROP_WILL_VALIDATE,
    PROP_OPTIONS,
    PROP_DISABLED,
    PROP_AUTOFOCUS,
    PROP_MULTIPLE,
    PROP_NAME,
    PROP_SIZE,
};


static void gdom_html_select_element_finalize(GObject* object)
{
    /*GdomHTMLSelectElement* request = GDOM_HTML_SELECT_ELEMENT(object);*/
    /*GdomHTMLSelectElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_HTML_ELEMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_select_element_parent_class)->finalize(object);
}

static void gdom_html_select_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLSelectElement *self = GDOM_HTML_SELECT_ELEMENT(object);
    WebCore::HTMLSelectElement * gself = (gdom_html_select_element_get_private(self));
    switch (prop_id) {
     case PROP_SELECTED_INDEX:
    {
         gself->setSelectedIndex((g_value_get_long (value)) );
         break;
    }
     case PROP_VALUE:
    {
         gself->setValue(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_LENGTH:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setLength((g_value_get_ulong (value)) , ec );
         break;
    }
     case PROP_DISABLED:
    {
         gself->setDisabled((g_value_get_boolean (value)) );
         break;
    }
     case PROP_AUTOFOCUS:
    {
         gself->setAutofocus((g_value_get_boolean (value)) );
         break;
    }
     case PROP_MULTIPLE:
    {
         gself->setMultiple((g_value_get_boolean (value)) );
         break;
    }
     case PROP_NAME:
    {
         gself->setName(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_SIZE:
    {
         gself->setSize((g_value_get_long (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_select_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLSelectElement *self = GDOM_HTML_SELECT_ELEMENT(object);
    WebCore::HTMLSelectElement * gself = (gdom_html_select_element_get_private(self));
    switch (prop_id) {
    case PROP_TYPE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->type()));
         break;
    }
    case PROP_SELECTED_INDEX:
    {
         g_value_set_long (value, gself->selectedIndex());
         break;
    }
    case PROP_VALUE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->value()));
         break;
    }
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    case PROP_FORM:
    {
         PassRefPtr<WebCore::HTMLFormElement> ptr = gself->form();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_WILL_VALIDATE:
    {
         g_value_set_boolean (value, gself->willValidate());
         break;
    }
    case PROP_OPTIONS:
    {
         PassRefPtr<WebCore::HTMLOptionsCollection> ptr = gself->options();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_DISABLED:
    {
         g_value_set_boolean (value, gself->disabled());
         break;
    }
    case PROP_AUTOFOCUS:
    {
         g_value_set_boolean (value, gself->autofocus());
         break;
    }
    case PROP_MULTIPLE:
    {
         g_value_set_boolean (value, gself->multiple());
         break;
    }
    case PROP_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->name()));
         break;
    }
    case PROP_SIZE:
    {
         g_value_set_long (value, gself->size());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_select_element_class_init(GdomHTMLSelectElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_select_element_finalize;
    g_klass->set_property = gdom_html_select_element__set_property_fn;
    g_klass->get_property = gdom_html_select_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_TYPE,
            g_param_spec_string ("type", /* name */
                "html_select_element_type", /* short description */
                "read-only  gchar * HTMLSelectElement.type", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SELECTED_INDEX,
            g_param_spec_long ("selected_index", /* name */
                "html_select_element_selected_index", /* short description */
                "read-write  glong HTMLSelectElement.selected_index", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_VALUE,
            g_param_spec_string ("value", /* name */
                "html_select_element_value", /* short description */
                "read-write  gchar * HTMLSelectElement.value", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "html_select_element_length", /* short description */
                "read-write  gulong HTMLSelectElement.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_FORM,
            g_param_spec_object ("form", /* name */
                "html_select_element_form", /* short description */
                "read-only  GdomHTMLFormElement * HTMLSelectElement.form", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_FORM_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WILL_VALIDATE,
            g_param_spec_boolean ("will_validate", /* name */
                "html_select_element_will_validate", /* short description */
                "read-only  gboolean HTMLSelectElement.will_validate", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OPTIONS,
            g_param_spec_object ("options", /* name */
                "html_select_element_options", /* short description */
                "read-only  GdomHTMLOptionsCollection * HTMLSelectElement.options", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_OPTIONS_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_DISABLED,
            g_param_spec_boolean ("disabled", /* name */
                "html_select_element_disabled", /* short description */
                "read-write  gboolean HTMLSelectElement.disabled", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_AUTOFOCUS,
            g_param_spec_boolean ("autofocus", /* name */
                "html_select_element_autofocus", /* short description */
                "read-write  gboolean HTMLSelectElement.autofocus", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_MULTIPLE,
            g_param_spec_boolean ("multiple", /* name */
                "html_select_element_multiple", /* short description */
                "read-write  gboolean HTMLSelectElement.multiple", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_NAME,
            g_param_spec_string ("name", /* name */
                "html_select_element_name", /* short description */
                "read-write  gchar * HTMLSelectElement.name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SIZE,
            g_param_spec_long ("size", /* name */
                "html_select_element_size", /* short description */
                "read-write  glong HTMLSelectElement.size", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLSelectElementPrivate));*/
}

static void gdom_html_select_element_init(GdomHTMLSelectElement* request)
{
    /*GdomHTMLSelectElementPrivate* priv = GDOM_HTML_SELECT_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

